
/*
 * code written by Mortiz v. Sivers, August 2020
 * needs Adafruit MPU6050 and FastLED libraries
 * 
 * 
===============================================
*/

#include <Adafruit_MPU6050.h>
#include <Adafruit_Sensor.h>
#include <Wire.h>

#define FASTLED_ALLOW_INTERRUPTS 0 //  simultaneuos use of I2C library may cause interrupt problems
#include "FastLED.h"

FASTLED_USING_NAMESPACE

Adafruit_MPU6050 mpu;

// normalized accelerations
float ax, ay, az, totalAccel;

// normalization constant for acceleration
double f;

#if defined(FASTLED_VERSION) && (FASTLED_VERSION < 3001000)
#warning "Requires FastLED 3.1 or later; check github for latest code."
#endif

#define DATA_PIN    6
#define LED_TYPE    WS2812B
#define NUM_LEDS    240       // total number of LEDs
CRGB leds[NUM_LEDS];
#define firstleftLED  121      // number of first LED on left side

#define BRIGHTNESS          128   // set master brightness to limit current draw

#define MotionDetThreshold 200    // shake detection threshold (0 .. 441)
#define standingThres -110        // detection threshold when surfboard is standing, normalized x-acceleration (-255 .. 0)
#define rightDetThres -30          // detection threshold for right turn, normalized y-acceleration (-255 .. 0)
#define leftDetThres 30          // detection threshold for left turn, normalized y-acceleration (0 .. 255)

void setup() {

  // initialize serial communication
  Serial.begin(9600);
  
    // Try to initialize!
  if (!mpu.begin()) {
    Serial.println("Failed to find MPU6050 chip");
    while (1) {
      delay(10);
    }
  }
  Serial.println("MPU6050 Found!");

  mpu.setAccelerometerRange(MPU6050_RANGE_2_G);
  Serial.print("Accelerometer range set to: ");
  switch (mpu.getAccelerometerRange()) {
  case MPU6050_RANGE_2_G:
    Serial.println("+-2G");
    f = 2*9.81;
    break;
  case MPU6050_RANGE_4_G:
    Serial.println("+-4G");
     f = 4*9.81;
    break;
  case MPU6050_RANGE_8_G:
    Serial.println("+-8G");
     f = 8*9.81;
    break;
  case MPU6050_RANGE_16_G:
    Serial.println("+-16G");
     f = 16*9.81;
    break;
  }
  mpu.setGyroRange(MPU6050_RANGE_250_DEG);
  Serial.print("Gyro range set to: ");
  switch (mpu.getGyroRange()) {
  case MPU6050_RANGE_250_DEG:
    Serial.println("+- 250 deg/s");
    break;
  case MPU6050_RANGE_500_DEG:
    Serial.println("+- 500 deg/s");
    break;
  case MPU6050_RANGE_1000_DEG:
    Serial.println("+- 1000 deg/s");
    break;
  case MPU6050_RANGE_2000_DEG:
    Serial.println("+- 2000 deg/s");
    break;
  }

  mpu.setFilterBandwidth(MPU6050_BAND_21_HZ);
  Serial.print("Filter bandwidth set to: ");
  switch (mpu.getFilterBandwidth()) {
  case MPU6050_BAND_260_HZ:
    Serial.println("260 Hz");
    break;
  case MPU6050_BAND_184_HZ:
    Serial.println("184 Hz");
    break;
  case MPU6050_BAND_94_HZ:
    Serial.println("94 Hz");
    break;
  case MPU6050_BAND_44_HZ:
    Serial.println("44 Hz");
    break;
  case MPU6050_BAND_21_HZ:
    Serial.println("21 Hz");
    break;
  case MPU6050_BAND_10_HZ:
    Serial.println("10 Hz");
    break;
  case MPU6050_BAND_5_HZ:
    Serial.println("5 Hz");
    break;
  }

    delay(1000); // 1 second delay for recovery
  
  // tell FastLED about the LED strip configuration
  FastLED.addLeds<LED_TYPE,DATA_PIN>(leds, NUM_LEDS).setCorrection(TypicalLEDStrip);

  // set master brightness control
  FastLED.setBrightness(BRIGHTNESS);
   
  FastLED.clear();
}


uint8_t gHue = 0; // rotating "base color" used by many of the patterns


void loop() {

  // read out gyroscope
  readGyro();

  // pumping animation when fast motion is detected
  if(totalAccel>MotionDetThreshold) {
    //Serial.println("motion detected");
    pumping();
  }
  else if(ax<standingThres) {
    standing();
  }
  else {
    detectTurn();
  }

   

// send the 'leds' array out to the actual LED strip
  FastLED.show();
  
  // do some periodic updates
  EVERY_N_MILLISECONDS( 20 ) { gHue++; } // slowly cycle the "base color" through the rainbow
    
}


void readGyro() {

  /* Get new sensor events with the readings */
  sensors_event_t a, g, temp;
  mpu.getEvent(&a, &g, &temp);

  // average 10 values with 1 ms delay
  ax = 0;
  ay = 0;
  az = 0;
  
  for (int i=0; i<10; i++) {
    ax += a.acceleration.x / f * 255;
    ay += a.acceleration.y / f * 255;
    az += a.acceleration.z / f * 255;
    delay(1);
  }
  ax /= 10;
  ay /= 10;
  az /= 10;
 
  totalAccel = sqrt(ax*ax+ay*ay+az*az);

  /* Print out the values */
  /*
  Serial.print("Acceleration X: ");
  Serial.print(ax);
  Serial.print(", Y: ");
  Serial.print(ay);
  Serial.print(", Z: ");
  Serial.println(az);

  Serial.print("total acceleration: ");
  Serial.println(totalAccel);

  Serial.print("Temperature: ");
  Serial.print(temp.temperature);
  Serial.println(" degC");
  */

}


// animation when a turn is detected
void detectTurn() {

  // for left turn ay>0
  if(ay>leftDetThres) {
    sinelon_left();
  }
  else if (ay<rightDetThres) {
    sinelon_right();
  }
  else {
    rainbowWithGlitter() ;
  }
  
}


// effect when standing
void standing() {
  //Serial.println("standing...");
  confetti();
}


// effect when pumping
void pumping() {
  fill_rainbow( leds, NUM_LEDS, gHue, 255/NUM_LEDS);
  FastLED.show();
  delay(200);
  fill_solid(leds, NUM_LEDS, CRGB::Black);
  FastLED.show();
  delay(200);
  fill_rainbow( leds, NUM_LEDS, gHue, 255/NUM_LEDS);
  FastLED.show();
  delay(200);
  fill_solid(leds, NUM_LEDS, CRGB::Black);
  FastLED.show();
  delay(200);
}


// FastLED's built-in rainbow generator
void rainbow()
{
  fill_rainbow( leds, NUM_LEDS, gHue, 255/NUM_LEDS);
}


// built-in FastLED rainbow, plus some random sparkly glitter
void rainbowWithGlitter() 
{
  rainbow();
  addGlitter(80);
}

void addGlitter( fract8 chanceOfGlitter) 
{
  if( random8() < chanceOfGlitter) {
    leds[ random16(NUM_LEDS) ] += CRGB::White;
  }
}

// random colored speckles that blink in and fade smoothly
void confetti() 
{
  fadeToBlackBy( leds, NUM_LEDS, 10);
  int pos = random16(NUM_LEDS);
  leds[pos] += CHSV( gHue + random8(64), 200, 255);
}

// a colored dot sweeping back and forth, with fading trails
void sinelon()
{
  
  fadeToBlackBy( leds, NUM_LEDS, 20);
  int pos = beatsin16( 13, 0, NUM_LEDS-1 );
  leds[pos] += CHSV( gHue, 255, 192);
}

// sinelon + dim right LEDs
void sinelon_left()
{
  for(int i=0; i<firstleftLED; ++i) {
      leds[i].fadeToBlackBy(64);
    }
    for(int i=firstleftLED; i<NUM_LEDS; ++i) {
      leds[i].fadeToBlackBy(20);
    }
  int pos = beatsin16( 13, firstleftLED, NUM_LEDS-1 );
  leds[pos] += CHSV( gHue, 255, 192);
}

// sinelon + dim left LEDs
void sinelon_right()
{
  // a colored dot sweeping back and forth, with fading trails
  for(int i=0; i<firstleftLED; ++i) {
      leds[i].fadeToBlackBy(20);
    }
    for(int i=firstleftLED; i<NUM_LEDS; ++i) {
      leds[i].fadeToBlackBy(64);
    }
  int pos = beatsin16( 13, 0, firstleftLED-1 );
  leds[pos] += CHSV( gHue, 255, 192);
}

// colored stripes pulsing at a defined Beats-Per-Minute (BPM)
void bpm()
{
  uint8_t BeatsPerMinute = 62;
  CRGBPalette16 palette = PartyColors_p;
  uint8_t beat = beatsin8( BeatsPerMinute, 64, 255);
  for( int i = 0; i < NUM_LEDS; i++) { //9948
    leds[i] = ColorFromPalette(palette, gHue+(i*2), beat-gHue+(i*10));
  }
}

// eight colored dots, weaving in and out of sync with each other
void juggle() {
  fadeToBlackBy( leds, NUM_LEDS, 20);
  byte dothue = 0;
  for( int i = 0; i < 8; i++) {
    leds[beatsin16( i+7, 0, NUM_LEDS-1 )] |= CHSV(dothue, 200, 255);
    dothue += 32;
  }
}
